@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 2.1.  */
+/* A Bison parser, made by GNU Bison 1.875c.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
 /* As a special exception, when this file is copied by Bison into a
    Bison output file, you may use that output file without restriction.
@@ -34,7 +34,7 @@
      COPYRIGHT = 260,
      CUSTOM = 261,
      DATE = 262,
-     DEBUG = 263,
+     DEBUG_K = 263,
      DESCRIPTION = 264,
      EXIT = 265,
      EXPORT = 266,
@@ -66,13 +66,12 @@
      QUOTED_STRING = 292
    };
 #endif
-/* Tokens.  */
 #define CHECK 258
 #define CODESTART 259
 #define COPYRIGHT 260
 #define CUSTOM 261
 #define DATE 262
-#define DEBUG 263
+#define DEBUG_K 263
 #define DESCRIPTION 264
 #define EXIT 265
 #define EXPORT 266
@@ -112,8 +111,8 @@ typedef union YYSTYPE {
   char *string;
   struct string_list *list;
 } YYSTYPE;
-/* Line 1447 of yacc.c.  */
-#line 117 "nlmheader.h"
+/* Line 1275 of yacc.c.  */
+#line 116 "nlmheader.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
