@@ -1,9 +1,7 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+/* A Bison parser, made by GNU Bison 1.875c.  */
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+/* Skeleton parser for Yacc-like parsing with Bison,
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,21 +15,13 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
-/* As a special exception, you may create a larger work that contains
-   part or all of the Bison parser skeleton and distribute that work
-   under terms of your choice, so long as that work isn't itself a
-   parser generator using the skeleton or a modified version thereof
-   as a parser skeleton.  Alternatively, if you modify or redistribute
-   the parser skeleton itself, you may (at your option) remove this
-   special exception, which will cause the skeleton and the resulting
-   Bison output files to be licensed under the GNU General Public
-   License without this special exception.
-
-   This special exception was added by the Free Software Foundation in
-   version 2.2 of Bison.  */
+/* As a special exception, when this file is copied by Bison into a
+   Bison output file, you may use that output file without restriction.
+   This special exception was added by the Free Software Foundation
+   in version 1.24 of Bison.  */
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -64,11 +54,11 @@
      NONAMEU = 280,
      NONAMEL = 281,
      DIRECTIVE = 282,
-     ID = 283,
-     DIGITS = 284
+     EQUAL = 283,
+     ID = 284,
+     DIGITS = 285
    };
 #endif
-/* Tokens.  */
 #define NAME 258
 #define LIBRARY 259
 #define DESCRIPTION 260
@@ -94,23 +84,22 @@
 #define NONAMEU 280
 #define NONAMEL 281
 #define DIRECTIVE 282
-#define ID 283
-#define DIGITS 284
+#define EQUAL 283
+#define ID 284
+#define DIGITS 285
 
 
 
 
-#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
+#if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
 #line 103 "deffilep.y"
-{
+typedef union YYSTYPE {
   char *id;
   int number;
   char *digits;
-}
-/* Line 1529 of yacc.c.  */
-#line 113 "deffilep.h"
-	YYSTYPE;
+} YYSTYPE;
+/* Line 1275 of yacc.c.  */
+#line 103 "deffilep.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -118,3 +107,5 @@ typedef union YYSTYPE
 
 extern YYSTYPE yylval;
 
+
+
